#include <string>
#include <limits>
#include <iostream>

#include "NN_factory.h"
#include "NN.h"

using namespace std;

NN_PSO &NN_factory::createFastestPSO(string nnetConfigFile, string trainingDataFile, string psoConfigFile)
{
    NN_PSO *nns[5];
    nns[0] = new NN_PSO(nnetConfigFile,trainingDataFile,psoConfigFile);
    nns[1] = new NN_PSO_OpenCL_CPU_local(nnetConfigFile,trainingDataFile,psoConfigFile);
    nns[2] = new NN_PSO_OpenCL_CPU_private(nnetConfigFile,trainingDataFile,psoConfigFile);
    nns[3] = new NN_PSO_OpenCL_GPU_local(nnetConfigFile,trainingDataFile,psoConfigFile);
    nns[4] = new NN_PSO_OpenCL_GPU_private(nnetConfigFile,trainingDataFile,psoConfigFile);


    // find fastest
    int idx=0; double tmin = numeric_limits<double>::max();

    for(int i=0;i<5;i++)
    {
        double t = nns[i]->timing();
        cout << "t[" << i << "]=" << t << endl;
        if(t < tmin)
        {
            tmin = t;
            idx = i;
        }
    }

    // delete everyone else
    for(int i=0;i<5;i++)
        if(i != idx) delete nns[i];

    return *nns[idx];
}


NN_PSO &NN_factory::createCPU_PSO(string nnetConfigFile, string trainingDataFile, string psoConfigFile)
{
    return *(new NN_PSO(nnetConfigFile,trainingDataFile,psoConfigFile)); // just wrong
}
NN_PSO &NN_factory::createGPU_PSO(string nnetConfigFile, string trainingDataFile, string psoConfigFile)
{
    return *(new NN_PSO(nnetConfigFile,trainingDataFile,psoConfigFile)); // just wrong
}
