#ifndef PSO_H
#define PSO_H

#include <string>
#include <vector>

class PsoParameters;


class ParticleSwarmOptimization
{
public:
    /** Constructor creates nParticles random particle position in range [boundaries[i].first, boundaries[i].second] for each dimension i (i = [0, boundaries.size()-1]).
        For each particle a speed is randomly created in range [-range, range] ( range = upper bound - lower bound).
        It works directly on */

    ParticleSwarmOptimization(PsoParameters& parameters, std::vector<float>& positions); // + PSO parameters
    std::vector<float>& getPositions();
    void updateFitnesses(std::vector<float> &fitnesses);
    float getSwarmFitness();
    std::vector<float>& getBestPosition();
    void nextIteration();

private:
public:


    void updateSpeeds();
    void moveParticles();

    // keeps a reference to PsoParameters object. The object is in NeuralNetworks.*/
    PsoParameters& params_;

    int iterations_;

    std::vector<float>& particlePositions_;
    std::vector<float> particleSpeeds_;

    std::vector<float> bestParticlePositions_;
    std::vector<float> bestParticleFitnesses_;
    std::vector<float> bestSwarmPosition_;
    float bestSwarmFitness_;

};


class PsoParameters
{
public:
    PsoParameters(std::string fileName);
    PsoParameters();

    void PrintInfo();

    int populationSize, nDimensions;
    float lowerLimit, upperLimit; // position limits for all dimension are the same
    float phiP, phiG, omega;
    float maxSpeed; // speed is in range [-maxSpeed,+maxSpeed] for all dimensions

    int stripLength; // calculate validation after every stripLength training calculations

    // terminating conditions
    int iterations;
    float maxFitness;
    float time;

private:

    void SetDefaults();

};


#endif //  PSO_H
